


const svg = d3.select("#place").append("svg")
  .attr("viewBox","0 0 960 480")
  .attr("preserveAspectRatio", "xMidYMid meet")
  .attr("stroke", "#AAAAAA");


// Queuing up asynchronous data loading. Convenient when you've got more than
// one file you want to load.
d3.queue()
  .defer(d3.json, "./scores.json")
  .defer(d3.json, "./world-50m.json")
  .await(ready);

// The ready function is called when our data files have finished loading.
// In this instance it takes 3 arguments:
// error -- which if will throw and error if there are problems with our data
// data -- which corresponds to the scores.json file
// map -- which contains the GeoJson mapping of the world's countries
function ready(error, data, map) {
  if (error) throw error;

  // call our map making function with the data we've loaded in from our scores.json file & our world map file
  WorldMap(data.location, map);
}


function WorldMap(data, map) {
  
  //extract parts of the data from our json file into simpler arrays
  let countryCodes = data.map( d => d.code );
  let countryNames = data.map( d => d.country );
  let countryTotals = data.map( d => d.total );

  //functions to transform country codes into colors, names, and totals
  let color = d3.scaleOrdinal().range(d3.schemeCategory20c).domain( countryCodes );
  
  
  let cnames = d3.scaleOrdinal().range(countryNames).domain(countryCodes);
  let ctotals = d3.scaleOrdinal().range(countryTotals).domain(countryCodes);

  //load in path
  let path = d3.geoPath().projection(d3.geoEquirectangular());
  //let path = d3.geoPath().projection(d3.geoMercator());

  //use topojson to create pathdata for each country in the world and overlay over our map paths
  let countries = topojson.feature(map, map.objects.countries).features;

  //placeholder for mouse coords, updated when we movemouse over a country
  let coordinates = [0, 0];


  //selects *all* countries and inserts the svg path for them, colors them light gray, and sets up mouse callback functions
  svg.selectAll(".country")
    .data(countries).enter()
    .insert("path")
    .attr("d", path)
    .style("fill", (d) => countryCodes.indexOf(d.id) > -1 ? color(d.id) : "#ccc" )
    .on('mouseover', overFunc)
    .on('mouseleave', outFunc);


  //handle mouseovers - just adds text for countries associated with data & turns country color white
  function overFunc(d, i) {

    //update mouse coordinates 
    coordinates = d3.mouse(this);

    //check if we are over a country that is in our json file
    d3.select(this).style('fill', "#fff");

    if (countryCodes.indexOf(d.id) > -1) {

      svg.selectAll(".country")
        .data(countries)
        .enter()
        .append("text")
        .text(         
            d2 => { //nice JS6 way 'arrow syntax' for doing anonymous functions
              if (countryCodes.indexOf(d2.id) == countryCodes.indexOf(d.id) ) {
                return cnames(d2.id) + " (" + ctotals(d2.id) + ")";
              } 
            }
            )
        .attr("x", coordinates[0])
        .attr("y", coordinates[1] - 10)
        .attr("text-anchor","middle")
        .attr("dy","-40")
        .attr('font-size','26px')
        .attr("fill", "black")
        .attr("stroke-width", "0px")
        .attr("font-family","Brown,helvetica,sans-serif") //I like this "Brown" font, if you don't have it, back up to helvetica or a sans-serif!
    }
  };


  //handle mouseleaves - removes text and returns to default color

  function outFunc(d, i) {

    d3.select(this).style('fill', countryCodes.indexOf(d.id) > -1 ? color(d.id) : "#ccc");

    if (countryCodes.indexOf(d.id) > -1) {
      svg.selectAll("text").remove();
    } 


  };




}

